@@ -1,11 +1,15 @@
 import sys
+import os
 from textwrap import dedent
 
 import py
 import pytest
 import tox
 import tox.config
-from tox.config import *  # noqa
+from tox.config import (
+    SectionReader, is_section_substitution, CommandParser,
+    parseconfig, DepOption, get_homedir, getcontextname,
+)
 from tox.venv import VirtualEnv
 
 
@@ -274,6 +278,34 @@ class TestIniParserAgainstCommandsKey:
             ["echo", "cmd", "1", "2", "3", "4", "cmd", "2"],
         ]
 
+    def test_command_substitution_from_other_section_posargs(self, newconfig):
+        """Ensure subsitition from other section with posargs succeeds"""
+        config = newconfig("""
+            [section]
+            key = thing {posargs} arg2
+            [testenv]
+            commands =
+                {[section]key}
+            """)
+        reader = SectionReader("testenv", config._cfg)
+        reader.addsubstitutions([r"argpos"])
+        x = reader.getargvlist("commands")
+        assert x == [['thing', 'argpos', 'arg2']]
+
+    def test_command_section_and_posargs_substitution(self, newconfig):
+        """Ensure subsitition from other section with posargs succeeds"""
+        config = newconfig("""
+            [section]
+            key = thing arg1
+            [testenv]
+            commands =
+                {[section]key} {posargs} endarg
+            """)
+        reader = SectionReader("testenv", config._cfg)
+        reader.addsubstitutions([r"argpos"])
+        x = reader.getargvlist("commands")
+        assert x == [['thing', 'arg1', 'argpos', 'endarg']]
+
     def test_command_env_substitution(self, newconfig):
         """Ensure referenced {env:key:default} values are substituted correctly."""
         config = newconfig("""
@@ -618,6 +650,55 @@ class TestIniParser:
         py.test.raises(tox.exception.ConfigError, 'reader.getbool("key5")')
 
 
+class TestIniParserPrefix:
+    def test_basic_section_access(self, tmpdir, newconfig):
+        config = newconfig("""
+            [p:section]
+            key=value
+        """)
+        reader = SectionReader("section", config._cfg, prefix="p")
+        x = reader.getstring("key")
+        assert x == "value"
+        assert not reader.getstring("hello")
+        x = reader.getstring("hello", "world")
+        assert x == "world"
+
+    def test_fallback_sections(self, tmpdir, newconfig):
+        config = newconfig("""
+            [p:mydefault]
+            key2=value2
+            [p:section]
+            key=value
+        """)
+        reader = SectionReader("section", config._cfg, prefix="p",
+                               fallbacksections=['p:mydefault'])
+        x = reader.getstring("key2")
+        assert x == "value2"
+        x = reader.getstring("key3")
+        assert not x
+        x = reader.getstring("key3", "world")
+        assert x == "world"
+
+    def test_value_matches_prefixed_section_substituion(self):
+        assert is_section_substitution("{[p:setup]commands}")
+
+    def test_value_doesn_match_prefixed_section_substitution(self):
+        assert is_section_substitution("{[p: ]commands}") is None
+        assert is_section_substitution("{[p:setup]}") is None
+        assert is_section_substitution("{[p:setup] commands}") is None
+
+    def test_other_section_substitution(self, newconfig):
+        config = newconfig("""
+            [p:section]
+            key = rue
+            [p:testenv]
+            key = t{[p:section]key}
+            """)
+        reader = SectionReader("testenv", config._cfg, prefix="p")
+        x = reader.getstring("key")
+        assert x == "true"
+
+
 class TestConfigTestEnv:
     def test_commentchars_issue33(self, tmpdir, newconfig):
         config = newconfig("""
@@ -1006,6 +1087,20 @@ class TestConfigTestEnv:
         argv = conf.commands
         assert argv[0] == ["echo"]
 
+    def test_substitution_double(self, newconfig):
+        inisource = """
+            [params]
+            foo = bah
+            foo2 = [params]foo
+
+            [testenv:py27]
+            commands =
+                echo {{[params]foo2}}
+        """
+        conf = newconfig([], inisource).envconfigs['py27']
+        argv = conf.commands
+        assert argv[0] == ['echo', 'bah']
+
     def test_posargs_backslashed_or_quoted(self, tmpdir, newconfig):
         inisource = """
             [testenv:py27]
@@ -1015,12 +1110,12 @@ class TestConfigTestEnv:
         """
         conf = newconfig([], inisource).envconfigs['py27']
         argv = conf.commands
-        assert argv[0] == ['echo', '\\{posargs\\}', '=']
+        assert argv[0] == ['echo', '{posargs}', '=']
         assert argv[1] == ['echo', 'posargs = ', ""]
 
         conf = newconfig(['dog', 'cat'], inisource).envconfigs['py27']
         argv = conf.commands
-        assert argv[0] == ['echo', '\\{posargs\\}', '=', 'dog', 'cat']
+        assert argv[0] == ['echo', '{posargs}', '=', 'dog', 'cat']
         assert argv[1] == ['echo', 'posargs = ', 'dog cat']
 
     def test_rewrite_posargs(self, tmpdir, newconfig):
@@ -1063,20 +1158,33 @@ class TestConfigTestEnv:
         argv = conf.commands
         assert argv[0] == ["cmd1", "hello"]
 
-    def test_take_dependencies_from_other_testenv(self, newconfig):
+    @pytest.mark.parametrize('envlist, deps', [
+        (['py27'], ('pytest', 'pytest-cov')),
+        (['py27', 'py34'], ('pytest', 'py{27,34}: pytest-cov')),
+    ])
+    def test_take_dependencies_from_other_testenv(
+        self,
+        newconfig,
+        envlist,
+        deps
+    ):
         inisource = """
+            [tox]
+            envlist = {envlist}
             [testenv]
-            deps=
-                pytest
-                pytest-cov
+            deps={deps}
             [testenv:py27]
             deps=
-                {[testenv]deps}
+                {{[testenv]deps}}
                 fun
-        """
+                frob{{env:ENV_VAR:>1.0,<2.0}}
+        """.format(
+            envlist=','.join(envlist),
+            deps='\n' + '\n'.join([' ' * 17 + d for d in deps])
+        )
         conf = newconfig([], inisource).envconfigs['py27']
         packages = [dep.name for dep in conf.deps]
-        assert packages == ['pytest', 'pytest-cov', 'fun']
+        assert packages == list(deps) + ['fun', 'frob>1.0,<2.0']
 
     def test_take_dependencies_from_other_section(self, newconfig):
         inisource = """
@@ -1410,10 +1518,10 @@ class TestGlobalOptions:
     def test_minversion(self, tmpdir, newconfig, monkeypatch):
         inisource = """
             [tox]
-            minversion = 3.0
+            minversion = 10.0
         """
-        config = newconfig([], inisource)
-        assert config.minversion == "3.0"
+        with py.test.raises(tox.exception.MinVersionError):
+            newconfig([], inisource)
 
     def test_skip_missing_interpreters_true(self, tmpdir, newconfig, monkeypatch):
         inisource = """
@@ -1837,6 +1945,23 @@ class TestCmdInvocation:
             "*ERROR*tox.ini*not*found*",
         ])
 
+    def test_override_workdir(self, tmpdir, cmd, initproj):
+        baddir = "badworkdir-123"
+        gooddir = "overridden-234"
+        initproj("overrideworkdir-0.5", filedefs={
+            'tox.ini': '''
+            [tox]
+            toxworkdir=%s
+            ''' % baddir,
+        })
+        result = cmd.run("tox", "--workdir", gooddir, "--showconfig")
+        assert not result.ret
+        stdout = result.stdout.str()
+        assert gooddir in stdout
+        assert baddir not in stdout
+        assert py.path.local(gooddir).check()
+        assert not py.path.local(baddir).check()
+
     def test_showconfig_with_force_dep_version(self, cmd, initproj):
         initproj('force_dep_version', filedefs={
             'tox.ini': '''
