@@ -5,6 +5,7 @@ import os
 import sys
 import tox.config
 from tox.venv import *  # noqa
+from tox.hookspecs import hookimpl
 from tox.interpreters import NoInterpreterInfo
 
 
@@ -167,7 +168,7 @@ def test_install_downloadcache(newmocksession, monkeypatch, tmpdir, envdc):
     assert len(l) == 2
     args = l[-1].args
     assert l[-1].cwd == venv.envconfig.config.toxinidir
-    assert "pip" in str(args[0])
+    assert "pip" in str(args)
     assert args[1] == "install"
     assert "dep1" in args
     assert "dep2" in args
@@ -262,6 +263,41 @@ def test_install_recreate(newmocksession, tmpdir):
     mocksession.report.expect("verbosity0", "*recreate*")
 
 
+def test_install_sdist_extras(newmocksession):
+    mocksession = newmocksession([], """
+        [testenv]
+        extras = testing
+            development
+    """)
+    venv = mocksession.getenv('python')
+    action = mocksession.newaction(venv, "getenv")
+    tox_testenv_create(action=action, venv=venv)
+    l = mocksession._pcalls
+    assert len(l) == 1
+    l[:] = []
+
+    venv.installpkg('distfile.tar.gz', action=action)
+    assert 'distfile.tar.gz[testing,development]' in l[-1].args
+
+
+def test_develop_extras(newmocksession, tmpdir):
+    mocksession = newmocksession([], """
+        [testenv]
+        extras = testing
+            development
+    """)
+    venv = mocksession.getenv('python')
+    action = mocksession.newaction(venv, "getenv")
+    tox_testenv_create(action=action, venv=venv)
+    l = mocksession._pcalls
+    assert len(l) == 1
+    l[:] = []
+
+    venv.developpkg(tmpdir, action=action)
+    expected = "%s[testing,development]" % tmpdir.strpath
+    assert expected in l[-1].args
+
+
 def test_test_hashseed_is_in_output(newmocksession):
     original_make_hashseed = tox.config.make_hashseed
     tox.config.make_hashseed = lambda: '123456789'
@@ -365,7 +401,7 @@ def test_install_python3(tmpdir, newmocksession):
     venv._install(["hello"], action=action)
     assert len(l) == 1
     args = l[0].args
-    assert 'pip' in str(args[0])
+    assert "pip" in args[0]
     for x in args:
         assert "--download-cache" not in args, args
 
@@ -672,3 +708,27 @@ def test_tox_testenv_create(newmocksession):
     venv = mocksession.getenv('python')
     venv.update(action=mocksession.newaction(venv, "getenv"))
     assert l == [1, 2]
+
+
+def test_tox_testenv_pre_post(newmocksession):
+    l = []
+
+    class Plugin:
+        @hookimpl
+        def tox_runtest_pre(self, venv):
+            l.append('started')
+
+        @hookimpl
+        def tox_runtest_post(self, venv):
+            l.append('finished')
+
+    mocksession = newmocksession([], """
+        [testenv]
+        commands=testenv_fail
+    """, plugins=[Plugin()])
+
+    venv = mocksession.getenv('python')
+    venv.status = None
+    assert l == []
+    mocksession.runtestenv(venv)
+    assert l == ['started', 'finished']
